/* Name: Wave Generator 1,999,999 MHz
   Description: Arduino Nano v3 ATmega328P + AD9833 DDS + ST7735 1,8" TFT + KY-040 Rotary Encoder
   Author:      TDM
   Compiler:    Arduino IDE 1.8.12
   Ver:         1.0b
   Date:        19/05/2020

       Arduino pinout diagram:
             _______________
            |      USB      |
            |           AREF|
            |            GND| 
            |             13| SCLK SPI ST7735/AD9833
            |RESET        12|
            |3V3          11| MOSI SPI ST7735/AD9833
            |5V           10| DC TFT ST7735/AD9833
            |GND           9|  
            |VIN           8| RESET TFT ST7735
            |               |
            |              7| CS TFT ST7735
            |              6| RELAY
            |A0 14         5|
            |A1 15         4|
 ENCODER S1 |A2 16         3| 
 ENCODER S2 |A3 17         2| ENCODER BUTTON
            |A4 18      TX 1|
            |A5 19      RX 0|
            |__A6_A7________|
*/

#include <SPI.h>
#include <Adafruit_GFX.h>    
#include <Adafruit_ST7735.h> 
#include <Fonts/FreeSans9pt7b.h>
#include <Fonts/FreeSans12pt7b.h> 
#include <Fonts/Picopixel.h>
#include <RotaryEncoder.h>

#define TFT_CS         7
#define TFT_RST        8
#define TFT_DC         10
#define ENC_BUTTON_PIN 2

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST);
RotaryEncoder encoder(A2, A3);

#define WHITE 0xFFFF      // RGB565 16 bit color
#define BLACK 0x0000                        
#define LIGHTGREY 0xC618
#define DARKGREY 0x7BEF     
#define GREY 0x73AE
#define NAVY 0x000F
#define DARKGREEN 0x03E0
#define DARKCYAN 0x03EF
#define MAROON 0x7800
#define PURPLE 0x780F
#define OLIVE 0x7BE0
#define BLUE 0x001F
#define GREEN 0x07E0
#define CYAN 0x07FF
#define RED 0xF800
#define MAGENTA 0xF81F
#define YELLOW 0xFFE0
#define ORANGE 0xFD20
#define GREENYELLOW 0xAFE5
#define PINK 0xF81F

uint32_t freq = 1000, freqOld = 1000;   // Частота по умолчанию 1кГц
char Str_Buffer[10];                    // Массив символов для вывода на LCD
char Str_Buffer_New[10];                // Массив для определения обновленных символов
uint8_t WaveForm = 3, WaveFormOld = 3, ButtonResultShort = 0, ButtonResultLong = 0;
volatile uint32_t buttonTimerStart;
volatile boolean buttonInterrupt = false;
int ButtonState = 0;
volatile int ButtonStateFlag = 0;
long milRead;
int MHz, kHz, Hz;
int SetDigit = 4, SetDigitOld = 4;
boolean FreqChangeFlag = false;
uint32_t freqChange = 1;
const float refFreq = 25000000.0;
int ChangeDigit; 
int pos = 0;
int newPos;

void setup() 
{
//  Serial.begin(9600); 
  SPI.begin();

  tft.initR(INITR_BLACKTAB);      // Инициализация ST7735S
  tft.setRotation(1);
  tft.setTextWrap(false);         
  tft.fillScreen(BLACK);

  tft.setTextColor(DARKGREY);
  tft.setTextSize(1);
  tft.setCursor(1, tft.height() -20);
  tft.println("AD9833 DDS GENERATOR");
  tft.print  ("    1 Hz to 2 MHz   ");

  SinIcon (125, 5, YELLOW);
  TriangleIcon (125, 45, LIGHTGREY);  
  SquareIcon (125, 85, LIGHTGREY);
  
  tft.setFont(&FreeSans12pt7b);
  tft.setTextColor(WHITE);
  tft.setCursor(10, 60);
  
  MHz = freq / 1000000;
  kHz = freq % 1000000 / 1000;
  Hz = freq % 1000;

  sprintf(Str_Buffer, "%1d,%03d,%03d", MHz, kHz, Hz);
  sprintf(Str_Buffer_New, "%1d,%03d,%03d", MHz, kHz, Hz);
  tft.print(Str_Buffer);
  tft.fillRoundRect(56,43,13,19,3,YELLOW);
  PaintDigit(SetDigit, BLACK);

  tft.drawFastVLine(10, 66, 4, DARKGREY);       
  tft.drawFastVLine(23, 66, 4, DARKGREY);
  tft.drawFastVLine(30, 66, 4, DARKGREY);
  tft.drawFastVLine(69, 66, 4, DARKGREY);
  tft.drawFastVLine(76, 66, 4, DARKGREY);
  tft.drawFastVLine(115,66, 4, DARKGREY);
  tft.drawFastHLine(10, 70, 14, DARKGREY);
  tft.drawFastHLine(30, 70, 40, DARKGREY);
  tft.drawFastHLine(76, 70, 40, DARKGREY);

  tft.setFont(&Picopixel);
  tft.setTextColor(DARKGREY);
  tft.setTextSize(2);
  tft.setCursor(3, 83);
  tft.print("MHz  kHz      Hz");
  tft.setTextSize(1);
  tft.setFont(&FreeSans12pt7b);

  AD9833SetFrequency(freq, WaveForm);
  
  cli();                                        // Глобальный запрет прерываний
  PCICR |= (1 << PCIE1);                        //
  PCMSK1 |= (1 << PCINT10) | (1 << PCINT11);    //
  EICRA |= (1 << ISC01) | (0 << ISC00);         // 
  EIMSK = (1 << INT0) | (0 << INT1);            // Разрешение прерываний INT0; ATmega328/P DATASHEET стр.90
  EIFR = 0x00;                                  // Сбрасываем флаги внешних прерываний; ATmega328/P DATASHEET стр.91
  sei();
}

void loop() 
{
  newPos = encoder.getPosition();
  if (pos != newPos)                            // Поверяем не изменилась ли позиция курсора
    {
      if (FreqChangeFlag == true)               // Проверяем вход в режим изменения частоты
        {
          sprintf(Str_Buffer, "%1d,%03d,%03d", MHz, kHz, Hz);
          SetNewFreq();                         // Обновляет значение частоты в переменной freq
          pos = newPos;
          
          MHz = freq / 1000000;
          kHz = freq % 1000000 / 1000;
          Hz = freq % 1000;
          sprintf(Str_Buffer_New, "%1d,%03d,%03d", MHz, kHz, Hz);

          if (Str_Buffer_New[0] != Str_Buffer[0]){if(SetDigit==7) {HighlightDigit(7,YELLOW);} else {tft.fillRect(10,43,13,19,BLACK); PaintDigit(7,WHITE);}}
          if (Str_Buffer_New[2] != Str_Buffer[2]){if(SetDigit==6) {HighlightDigit(6,YELLOW);} else {tft.fillRect(30,43,13,19,BLACK); PaintDigit(6,WHITE);}}
          if (Str_Buffer_New[3] != Str_Buffer[3]){if(SetDigit==5) {HighlightDigit(5,YELLOW);} else {tft.fillRect(43,43,13,19,BLACK); PaintDigit(5,WHITE);}}
          if (Str_Buffer_New[4] != Str_Buffer[4]){if(SetDigit==4) {HighlightDigit(4,YELLOW);} else {tft.fillRect(56,43,13,19,BLACK); PaintDigit(4,WHITE);}}
          if (Str_Buffer_New[6] != Str_Buffer[6]){if(SetDigit==3) {HighlightDigit(3,YELLOW);} else {tft.fillRect(76,43,13,19,BLACK); PaintDigit(3,WHITE);}}
          if (Str_Buffer_New[7] != Str_Buffer[7]){if(SetDigit==2) {HighlightDigit(2,YELLOW);} else {tft.fillRect(89,43,13,19,BLACK); PaintDigit(2,WHITE);}}
          if (Str_Buffer_New[8] != Str_Buffer[8]){if(SetDigit==1) {HighlightDigit(1,YELLOW);} else {tft.fillRect(102,43,13,19,BLACK);PaintDigit(1,WHITE);}}
           
          AD9833SetFrequency(freq, WaveForm); 
        }  
      else
        {
          if (newPos > pos) SetDigit++;
          else if (newPos < pos) SetDigit--;
          if (SetDigit < 1) SetDigit = 7;                
          if (SetDigit > 7) SetDigit = 1;
          pos = newPos;    
        }
    }       
  if (SetDigit != SetDigitOld)                            // Если позиция курсора изменена подсвечиваем новую позицию
    { 
      HighlightDigitClear(SetDigit, YELLOW);
      SetDigitOld = SetDigit;         
    }          
  switch (ButtonCheck())                                  // Проверяем нажатие кнопки
    {
      case 0: break;                                      // Ложная тревога
      case 1: FreqChangeFlag = !FreqChangeFlag; break;    // Детектировано короткое нажатие
      case 2:                                             // Детектировано длинное нажатие
          WaveForm++;
          if (WaveForm == 0) WaveForm = 1;
          if (WaveForm > 3) WaveForm = 1; 
          ChangeWaveForm(WaveForm);      
          break;
    }                       
}

uint8_t ButtonCheck()
{
  #define ShortPush 50
  #define LongPush  1000
  if (buttonInterrupt == true)
    {
      if (digitalRead(2) == LOW)
        {
         long milRead = millis();
         if (milRead - buttonTimerStart >= ShortPush && milRead - buttonTimerStart < LongPush && ButtonStateFlag == 0) 
          {
            ButtonStateFlag = 1; 
            return 1;
          }
         else if (milRead - buttonTimerStart >= LongPush && ButtonStateFlag == 1) 
          {
            ButtonStateFlag = 2;
            return 2;
          }
         else if (ButtonStateFlag == 2) 
          {
            ButtonStateFlag = 0;
            buttonInterrupt = false; 
            EIMSK |= 1 << INT0; 
            return 0;
          }
         else return 0;
        }
      else 
        {
          buttonInterrupt = false; 
          EIMSK |= 1 << INT0; 
          return 0;
        }
     }
}

ISR(INT0_vect)      // Вектор прерывания от кнопки энкодера
  {
    ButtonStateFlag = 0;
    buttonTimerStart = millis();
    buttonInterrupt = true;
    EIMSK = 0 << INT0;
  }

ISR(PCINT1_vect)
  {
    encoder.tick();   // just call tick() to check the state.
  }

void ChangeWaveForm(uint8_t WaveType) // Меняет тип сигнала на выходе, изменяет вид значка сигнала
{
  switch (WaveType)
    {
      case 3:  
        SinIcon (125, 5, YELLOW);
        TriangleIcon (125, 45, LIGHTGREY);  
        SquareIcon (125, 85, LIGHTGREY);  
        AD9833SetWaveform(0x2000); 
      break;
      case 1:  
        SinIcon (125, 5, LIGHTGREY);
        TriangleIcon (125, 45, YELLOW);  
        SquareIcon (125, 85, LIGHTGREY); 
        AD9833SetWaveform(0x2002);
      break;
      case 2:  
         SinIcon (125, 5, LIGHTGREY);
         TriangleIcon (125, 45, LIGHTGREY);  
         SquareIcon (125, 85, YELLOW);
         AD9833SetWaveform(0x2028); 
      break;
     }   
}

void AD9833SetWaveform(uint16_t WaveformCode) 
{
  AD9833WriteRegister(0x2100);   
  AD9833WriteRegister(WaveformCode);             // Exit & Reset to SINE, SQUARE or TRIANGLE
}

void AD9833SetFrequency(long frequency, uint8_t WaveType) 
{
  long FreqWord = (frequency * pow(2, 28)) / refFreq;
  int MSB = (int)((FreqWord & 0xFFFC000) >> 14);      //Only lower 14 bits are used for data
  int LSB = (int)(FreqWord & 0x3FFF);
  LSB |= 0x4000;                                      //Set control bits 15 ande 14 to 0 and 1, respectively, for frequency register 0
  MSB |= 0x4000; 
  AD9833WriteRegister(0x2100);   
  AD9833WriteRegister(LSB);                  // Write lower 16 bits to AD9833 registers
  AD9833WriteRegister(MSB);                  // Write upper 16 bits to AD9833 registers.
  switch (WaveType)
    {
      case 1: AD9833WriteRegister(0x2002); break;
      case 2: AD9833WriteRegister(0x2028); break;
      case 3: AD9833WriteRegister(0x2000); break;
    }
}

void AD9833WriteRegister(int dat) 
{ 
  SPI.setDataMode(SPI_MODE2);         // Display and AD9833 use different SPI MODES so it has to be set for the AD9833 here.
  digitalWrite(TFT_DC, LOW);          // Set FSYNC low before writing to AD9833 registers
  delayMicroseconds(10);              // Give AD9833 time to get ready to receive data.
  SPI.transfer(highByte(dat));        // Each AD9833 register is 32 bits wide and each 16
  SPI.transfer(lowByte(dat));         // bits has to be transferred as 2 x 8-bit bytes.
  digitalWrite(TFT_DC, HIGH);         // Write done. Set FSYNC high
}

void HighlightDigitClear(uint8_t NewPosition, uint16_t color) // Выделяет символ по указанной позиции, стирает старое выделение, перерисовывает старый символ
{
  static uint8_t OldPosition = 4;
  if (OldPosition!=NewPosition)
    {  
      switch (NewPosition) 
        {
          case 1: tft.fillRoundRect(102,43,13,19,3,color);PaintDigit(NewPosition, BLACK); break;
          case 2: tft.fillRoundRect(89,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 3: tft.fillRoundRect(76,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 4: tft.fillRoundRect(56,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 5: tft.fillRoundRect(43,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 6: tft.fillRoundRect(30,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 7: tft.fillRoundRect(10,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
        }
      switch (OldPosition) 
        {
          case 1: tft.fillRect(102,43,13,19,BLACK);PaintDigit(OldPosition, WHITE); break;
          case 2: tft.fillRect(89,43,13,19,BLACK); PaintDigit(OldPosition, WHITE); break;
          case 3: tft.fillRect(76,43,13,19,BLACK); PaintDigit(OldPosition, WHITE); break;
          case 4: tft.fillRect(56,43,13,19,BLACK); PaintDigit(OldPosition, WHITE); break;
          case 5: tft.fillRect(43,43,13,19,BLACK); PaintDigit(OldPosition, WHITE); break;
          case 6: tft.fillRect(30,43,13,19,BLACK); PaintDigit(OldPosition, WHITE); break;
          case 7: tft.fillRect(10,43,13,19,BLACK); PaintDigit(OldPosition, WHITE); break;
        }
      OldPosition = NewPosition;
    }
}

void HighlightDigit(uint8_t NewPosition, uint16_t color)  // Выделяет символ по указанной позиции
{
      switch (NewPosition) 
        {
          case 1: tft.fillRoundRect(102,43,13,19,3,color);PaintDigit(NewPosition, BLACK); break;
          case 2: tft.fillRoundRect(89,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 3: tft.fillRoundRect(76,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 4: tft.fillRoundRect(56,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 5: tft.fillRoundRect(43,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 6: tft.fillRoundRect(30,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
          case 7: tft.fillRoundRect(10,43,13,19,3,color); PaintDigit(NewPosition, BLACK); break;
        }
}

void PaintDigit(uint8_t DigitPos, uint16_t color) // Рисует символ по указанной позиции исходя и текущей частоты
{  
  tft.setTextColor(color);
  switch (DigitPos)
    {
      case 1: 
        ChangeDigit = freq % 10;  
        tft.setCursor(102, 60);
        tft.print(ChangeDigit);
      break;  
      case 2: 
        ChangeDigit = (freq / 10) % 10; 
        tft.setCursor(89, 60);
        tft.print(ChangeDigit);
      break;
      case 3: 
        ChangeDigit = (freq / 100) % 10;
        tft.setCursor(76, 60);
        tft.print(ChangeDigit);
      break;
      case 4: 
        ChangeDigit = (freq / 1000) % 10;
        tft.setCursor(56, 60);
        tft.print(ChangeDigit);
      break;
      case 5: 
        ChangeDigit = (freq / 10000) % 10; 
        tft.setCursor(43, 60);
        tft.print(ChangeDigit);
      break;
      case 6: 
        ChangeDigit = (freq / 100000) % 10;
        tft.setCursor(30, 60);
        tft.print(ChangeDigit);
      break;
      case 7: 
        ChangeDigit = freq / 1000000; 
        tft.setCursor(10, 60);
        tft.print(ChangeDigit);
      break;
    }                   
}

void SetNewFreq() // Вычисляет новое значение частоты исходя из поворота энкодера и с учетом ограничения
{  
    switch (SetDigit)
    {
      case 1:  
      freqChange = 1; 
      if (freq / freqChange < 1999999 && freq / freqChange > 0)
        {
          if (newPos < pos) freq = freq - freqChange;
          else if (newPos > pos) freq = freq + freqChange;
        }
      else if (freq / freqChange == 1999999 && newPos < pos) freq = freq - freqChange;
      if (freq / freqChange == 0 && newPos > pos) freq = freq + freqChange; 
      break;
      case 2:  
      freqChange = 10; 
      if (freq / freqChange < 199999 && freq / freqChange > 0)
        {
          if (newPos < pos) freq = freq - freqChange;
          else if (newPos > pos) freq = freq + freqChange;
        }
      else if (freq / freqChange == 199999 && newPos < pos) freq = freq - freqChange;
      if (freq / freqChange == 0 && newPos > pos) freq = freq + freqChange; 
      break;
      case 3:  
      freqChange = 100; 
      if (freq / freqChange < 19999 && freq / freqChange > 0)
        {
          if (newPos < pos) freq = freq - freqChange;
          else if (newPos > pos) freq = freq + freqChange;
        }
      else if (freq / freqChange == 19999 && newPos < pos) freq = freq - freqChange;
      if (freq / freqChange == 0 && newPos > pos) freq = freq + freqChange; 
      break;     
      case 4:  
      freqChange = 1000; 
      if (freq / freqChange < 1999 && freq / freqChange > 0)
        {
          if (newPos < pos) freq = freq - freqChange;
          else if (newPos > pos) freq = freq + freqChange;
        }
      else if (freq / freqChange == 1999 && newPos < pos) freq = freq - freqChange;
      if (freq / freqChange == 0 && newPos > pos) freq = freq + freqChange; 
      break;
      case 5:  
      freqChange = 10000; 
      if (freq / freqChange < 199 && freq / freqChange > 0)
        {
          if (newPos < pos) freq = freq - freqChange;
          else if (newPos > pos) freq = freq + freqChange;
        }
      else if (freq / freqChange == 199 && newPos < pos) freq = freq - freqChange;
      if (freq / freqChange == 0 && newPos > pos) freq = freq + freqChange; 
      break;
      case 6:  
      freqChange = 100000; 
      if (freq / freqChange < 19 && freq / freqChange > 0)
        {
          if (newPos < pos) freq = freq - freqChange;
          else if (newPos > pos) freq = freq + freqChange;
        }
      else if (freq / freqChange == 19 && newPos < pos) freq = freq - freqChange;
      if (freq / freqChange == 0 && newPos > pos) freq = freq + freqChange; 
      break;
      case 7:  
      freqChange = 1000000; 
      if (freq / freqChange < 1 && freq / freqChange > 0)
        {
          if (newPos < pos) freq = freq - freqChange;
          else if (newPos > pos) freq = freq + freqChange;
        }
      else if (freq / freqChange == 1 && newPos < pos) freq = freq - freqChange;
      if (freq / freqChange == 0 && newPos > pos) freq = freq + freqChange; 
      break;    
    }
}

void SquareIcon (uint8_t SX, uint8_t SY, uint16_t Color)
{
  tft.drawRoundRect(SX, SY, 32, 32, 7, Color);
  tft.drawRoundRect(SX+1, SY+1, 30, 30, 5, Color); 
  for (uint16_t a=0; a<2; a++)
  {    tft.drawFastVLine(SX+4+a, SY+6, 12, Color);}    
  for (uint16_t a=0; a<2; a++)
  {    tft.drawFastHLine(SX+4, SY+6+a, 11, Color);} 
  for (uint16_t a=0; a<2; a++)
  {    tft.drawFastVLine(SX+15+a, SY+6, 20, Color);}     
  for (uint16_t a=0; a<2; a++)
  {    tft.drawFastHLine(SX+15, SY+24+a, 12, Color);} 
  for (uint16_t a=0; a<2; a++)
  {    tft.drawFastVLine(SX+26+a, SY+14, 12, Color);}   
}

void TriangleIcon (uint8_t SX, uint8_t SY, uint16_t Color)
{
  tft.drawRoundRect(SX, SY, 32, 32, 7, Color);
  tft.drawRoundRect(SX+1, SY+1, 30, 30, 5, Color);
  for (uint16_t a=0; a<3; a++)
    {tft.drawLine(SX+9, SY+3+a, SX+3, SY+14+a, Color);}   
  for (uint16_t a=0; a<3; a++)
    {tft.drawLine(SX+21, SY+26+a, SX+9, SY+3+a, Color);}   
  for (uint16_t a=0; a<3; a++)
    {tft.drawLine(SX+28, SY+15+a, SX+21, SY+26+a, Color);}   
}

void SinIcon (uint8_t SX, uint8_t SY, uint16_t Color)
{
uint16_t Sine_Icon_only[676] PROGMEM={
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0010 (16) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color,   // 0x0020 (32) pixels
Color, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0030 (48) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, Color, Color, Color, 0x0000, 0x0000,   // 0x0040 (64) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0050 (80) pixels
0x0000, Color, Color, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0060 (96) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000,   // 0x0070 (112) pixels
0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0080 (128) pixels
0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000,   // 0x0090 (144) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color,   // 0x00A0 (160) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x00B0 (176) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x00C0 (192) pixels
Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x00D0 (208) pixels
0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color, 0x0000, 0x0000, 0x0000,   // 0x00E0 (224) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000,   // 0x00F0 (240) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0100 (256) pixels
0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color,   // 0x0110 (272) pixels
Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color,   // 0x0120 (288) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0130 (304) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0140 (320) pixels
0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0150 (336) pixels
Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color,   // 0x0160 (352) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0170 (368) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0180 (384) pixels
0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0190 (400) pixels
0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color,   // 0x01A0 (416) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000,   // 0x01B0 (432) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x01C0 (448) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x01D0 (464) pixels
0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x01E0 (480) pixels
0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color, 0x0000, 0x0000, 0x0000,   // 0x01F0 (496) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color, 0x0000,   // 0x0200 (512) pixels
0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0210 (528) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color,   // 0x0220 (544) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0230 (560) pixels
0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0240 (576) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, 0x0000, 0x0000,   // 0x0250 (592) pixels
0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0260 (608) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color, 0x0000, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0270 (624) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0280 (640) pixels
0x0000, Color, Color, Color, Color, Color, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,   // 0x0290 (656) pixels
0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, Color, Color, Color, 0x0000,   // 0x02A0 (672) pixels
};
  tft.drawRoundRect(SX, SY, 32, 32, 7, Color);
  tft.drawRoundRect(SX+1, SY+1, 30, 30, 5, Color);  
  tft.drawRGBBitmap(SX+3, SY+3, Sine_Icon_only, 26, 26);
}
